unit Error;

interface

uses
  Forms, Windows, SysUtils, COMObj, Dialogs;

type
  TError = class(TObject)
  private
    FOldOnException : TExceptionEvent;
  public
    constructor Create;
    destructor Destroy; override;  
    function DisplayException(Module, Prefix : String; E : TObject; ExceptAddr : Pointer) : HResult;
    procedure OnException(Sender: TObject; E: Exception);
  end;

var
  AnError : TError;

implementation

constructor TError.Create;
begin
  inherited Create;
  FOldOnException := Application.OnException;
  Application.OnException := OnException;
end;

destructor TError.Destroy;
begin
  Application.OnException := FOldOnException;
  inherited Destroy;
end;

function TError.DisplayException(Module, Prefix : String; E : TObject; ExceptAddr : Pointer) : HResult;
begin
  if E is EOLESysError then
    Result := (EOLESysError(E)).ErrorCode
  else if E is Exception then
    begin
      MessageDlg(Module + #13 + Prefix + #13 + Exception(E).Message,mtError,[mbOK],0);
      Result := S_OK;
    end
  else
     // Chances are good that if it gets to here, either a non-exception object
     // was raised, or the exception was raised in another DLL and bled through
     // to this one.
    Result := E_UNEXPECTED;
end;

procedure TError.OnException(Sender : TObject; E: Exception);
begin 
  OleCheck(DisplayException(Application.Title,'',E,ExceptAddr));
end;

initialization
  AnError := TError.Create;
finalization
  AnError.Free;
end.
