unit Main;

interface

uses
  Windows, ActiveX, COMObj, UOScrInter, Settings;

const
  PlugInTitle = 'Blackthorn''s Retreat';
  PlugInName = 'BlackthornsRetreat';
  CLSID : TGUID = '{115F702A-9573-11D1-88C5-00C04FB936DE}';
  DragonStart : Array[0..2] of TPoint =
    ((X: 900; Y: 300), (X: 1150; Y: 250), (X: 1100; Y: 380));

type
  TMinimalCharacter = record
    DeltaX : Integer;
    CharStruct : PCharacterStruct;
  end;

  TThisPlugIn = class (TCOMObject, IUOScrPlugIn)
  private
    Parent : IUOScr;
    frmSettings: TfrmSettings;

     //
    FrameCounter : Integer;

     // Characters
    BlackThorn : TMinimalCharacter;
    Dragons : Array[0..2] of TMinimalCharacter;
  public

     // IUOScrPlugIn
    function Activate(AParent : IUOScr): HResult; stdcall;
    function Deactivate: HResult; stdcall;
    function DoFrame: HResult; stdcall;
    function DoSettings : HResult; stdcall;
  end;

function DllOverrideRegisterServer: HResult; stdcall;
function DllOverrideUnRegisterServer: HResult; stdcall;

implementation

uses
  COMServ, SysUtils, Registry, Error;

function BlockExceptions: HResult;
begin
   // There should be no need to change this routine.
  Result := AnError.DisplayException(PlugInTitle, '', ExceptObject, ExceptAddr);
end;

function TThisPlugIn.Activate(AParent : IUOScr): HResult; stdcall;
var
  I : Integer;
begin
  Result := S_OK;
  try
    Parent := AParent;

     //
     // Allocate characters
     //

     // Blackthorn
    OleCheck(Parent.NeedCharacter(138425, 35,  640, 300, 0, Blackthorn.CharStruct));
    Blackthorn.DeltaX := -3;
    Blackthorn.CharStruct.FrameSet := 2;

     // The dragons
    For I := 0 to 2 do
      begin
        OleCheck(Parent.NeedCharacter(  6490, 22,  DragonStart[I].X, DragonStart[I].Y, 0, Dragons[I].CharStruct));
        with Dragons[I] do
          begin
            DeltaX := -3;
            CharStruct.FrameSet := 19;
            CharStruct.Frame := Random(8);
          end;
      end;
  except Result := BlockExceptions; end;
end;

function TThisPlugIn.Deactivate: HResult; stdcall;
var
  I : Integer;
begin
  Result := S_OK;
  try

     //
     // Release characters
     //
    for I := 2 downto 0 do
      Parent.ReleaseCharacter(Dragons[I].CharStruct);
    Parent.ReleaseCharacter(Blackthorn.CharStruct);
    Parent := Nil;
  except Result := BlockExceptions; end;
end;

procedure MoveChar(var Ch : TMinimalCharacter);
begin
  Inc(Ch.CharStruct.X, Ch.DeltaX);

  if Ch.DeltaX < 0 then
    begin
      if Ch.CharStruct.X < -400 then
        begin
          Ch.DeltaX := -Ch.DeltaX; // Turn around
          Ch.CharStruct.Direction := dirRight;
        end;
    end
  else
    begin
      if Ch.CharStruct.X > 1040 then
        begin
          Ch.DeltaX := -Ch.DeltaX; // Turn around
          Ch.CharStruct.Direction := dirLeft;
        end;
    end;
end;

function TThisPlugIn.DoFrame: HResult; stdcall;
begin
  Result := S_OK;
  try
    Inc(FrameCounter);

    MoveChar(Blackthorn);
    MoveChar(Dragons[0]);
    MoveChar(Dragons[1]);
    MoveChar(Dragons[2]);

    if FrameCounter and $03 = $03 then
      Inc(Blackthorn.CharStruct.Frame);
    if FrameCounter and $07 = $07 then
      begin
        Inc(Dragons[0].CharStruct.Frame);
        Inc(Dragons[1].CharStruct.Frame);
        Inc(Dragons[2].CharStruct.Frame);
      end;
  except Result := BlockExceptions; end;
end;

function TThisPlugIn.DoSettings : HResult; stdcall;
begin
   // There should be no need to change this routine.
  Result := S_OK;
  try
    _AddRef;
    try
      frmSettings := TfrmSettings.Create(Nil);
      frmSettings.ShowModal;
    finally
      _Release;
    end;
  except Result := BlockExceptions; end;
end;

 //-----------------------------------------------------------------------------

function DllOverrideRegisterServer: HResult;
var
  Reg : TRegistry;
begin
   // There should be no need to change this routine.
  Result := S_OK;
  try
    OleCheck(ComServ.DllRegisterServer);

    Reg := TRegistry.Create;
    try
      if not Reg.OpenKey(SPlugInRegKey + PlugInTitle, True) then
        raise Exception.Create('Error registering Plug-In (1)');
      if not Reg.OpenKey(SPlugInRegKey + PlugInTitle + '\CLSID', True) then
        raise Exception.Create('Error registering Plug-In (2)');
      Reg.WriteString('', GUIDToString(CLSID));
    finally
      Reg.Free;
    end;
  except Result := BlockExceptions; end;
end;

function DllOverrideUnRegisterServer: HResult;
var
  Reg : TRegistry;
begin
   // There should be no need to change this routine.
  Result := S_OK;
  try
    OleCheck(ComServ.DllUnRegisterServer);

    Reg := TRegistry.Create;
    try
      if not Reg.DeleteKey(SPlugInRegKey + PlugInTitle + '\CLSID') then
        raise Exception.Create('Error unregistering Plug-In (2)');
      if not Reg.DeleteKey(SPlugInRegKey + PlugInTitle) then
        raise Exception.Create('Error unregistering Plug-In (1)');
    finally
      Reg.Free;
    end;
  except Result := BlockExceptions; end;
end;

initialization
  ComServer.SetServerName('UOScr.PlugIn');
  TComObjectFactory.Create(ComServer, TThisPlugIn, CLSID,
       PlugInName, PlugInTitle, ciMultiInstance);
end.
